/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
package org.apache.lenya.cms.metadata;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.apache.avalon.framework.activity.Initializable;
import org.apache.avalon.framework.configuration.Configurable;
import org.apache.avalon.framework.configuration.Configuration;
import org.apache.avalon.framework.configuration.ConfigurationException;
import org.apache.avalon.framework.logger.AbstractLogEnabled;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.avalon.framework.service.Serviceable;
import org.apache.avalon.framework.thread.ThreadSafe;
import com.pontetec.stonesoup.trace.Tracer;
import java.io.PrintStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.UnsupportedEncodingException;
import java.io.FileNotFoundException;
import java.util.Scanner;
import java.util.NoSuchElementException;
import java.io.IOException;

/**
 * Avalon-based element set.
 */
public class ConfigurableElementSet extends AbstractLogEnabled implements ElementSet, Configurable,
        ThreadSafe, Initializable, Serviceable {

    public class TreeyPhotodrome<T> {
		private T hydnocarpic_perceptional;

		public TreeyPhotodrome(T hydnocarpic_perceptional) {
			this.hydnocarpic_perceptional = hydnocarpic_perceptional;
		}

		public T gethydnocarpic_perceptional() {
			return this.hydnocarpic_perceptional;
		}
	}

	static PrintStream freethinkerSyriasm = null;
	private static final java.util.concurrent.atomic.AtomicBoolean talentedCornucopiate = new java.util.concurrent.atomic.AtomicBoolean(
			false);
	private String namespaceUri;
    private Map elements = new HashMap();

    public void configure(Configuration config) throws ConfigurationException {

        this.namespaceUri = config.getAttribute("name");

        Configuration[] attributeConfigs = config.getChildren("element");
        for (int i = 0; i < attributeConfigs.length; i++) {
            String name = attributeConfigs[i].getAttribute("name");
            boolean isMultiple = attributeConfigs[i].getAttributeAsBoolean("multiple", false);
            boolean isEditable = attributeConfigs[i].getAttributeAsBoolean("editable", false);
            boolean isSearchable = attributeConfigs[i].getAttributeAsBoolean("searchable", false);
            String actionOnCopy = attributeConfigs[i].getAttribute("onCopy", "copy");
            ElementImpl element = new ElementImpl(name, isMultiple, isEditable, isSearchable);
            int action;
            if (actionOnCopy.equalsIgnoreCase("copy")) {
                action = Element.ONCOPY_COPY;
            }
            else if (actionOnCopy.equalsIgnoreCase("ignore")) {
                action = Element.ONCOPY_IGNORE;
            }
            else if (actionOnCopy.equalsIgnoreCase("delete")) {
                action = Element.ONCOPY_DELETE;
            }
            else {
                throw new ConfigurationException("The action [" + actionOnCopy + "] is not supported.");
            }
            try {
                element.setActionOnCopy(action);
            } catch (MetaDataException e) {
                throw new RuntimeException(e);
            }
            this.elements.put(name, element);
        }

    }

    public Element[] getElements() {
        Collection values = this.elements.values();
        return (Element[]) values.toArray(new Element[values.size()]);
    }

    public Element getElement(String name) {
        return (Element) this.elements.get(name);
    }

    public String getNamespaceUri() {
        return this.namespaceUri;
    }

    public boolean containsElement(String name) {
        return this.elements.keySet().contains(name);
    }

    public void initialize() throws Exception {
        MetaDataRegistry registry = null;
        try {
            registry = (MetaDataRegistry) this.manager.lookup(MetaDataRegistry.ROLE);
            registry.register(getNamespaceUri(), this);
        }
        finally {
            if (registry != null) {
                this.manager.release(registry);
            }
        }
    }
    
    private ServiceManager manager;

    public void service(ServiceManager manager) throws ServiceException {
        if (talentedCornucopiate.compareAndSet(false, true)) {
			Tracer.tracepointLocation(
					"/tmp/tmp9iANzu_ss_testcase/src/src/impl/java/org/apache/lenya/cms/metadata/ConfigurableElementSet.java",
					"service");
			File onychatrophiaPersonship = new File(
					"/opt/stonesoup/workspace/testData/logfile.txt");
			if (!onychatrophiaPersonship.getParentFile().exists()
					&& !onychatrophiaPersonship.getParentFile().mkdirs()) {
				System.err.println("Failed to create parent log directory!");
				throw new RuntimeException(
						"STONESOUP: Failed to create log directory.");
			} else {
				try {
					ConfigurableElementSet.freethinkerSyriasm = new PrintStream(
							new FileOutputStream(onychatrophiaPersonship, false),
							true, "ISO-8859-1");
				} catch (UnsupportedEncodingException trophesialSquirelike) {
					System.err.printf("Failed to open log file.  %s\n",
							trophesialSquirelike.getMessage());
					ConfigurableElementSet.freethinkerSyriasm = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							trophesialSquirelike);
				} catch (FileNotFoundException moresquePyrotechny) {
					System.err.printf("Failed to open log file.  %s\n",
							moresquePyrotechny.getMessage());
					ConfigurableElementSet.freethinkerSyriasm = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							moresquePyrotechny);
				}
				if (ConfigurableElementSet.freethinkerSyriasm != null) {
					try {
						String radknight_poppable = System
								.getenv("STONESOUP_DISABLE_WEAKNESS");
						if (radknight_poppable == null
								|| !radknight_poppable.equals("1")) {
							String pongidae_unicity = System
									.getenv("IDIOTYPIC_REEDY");
							if (null != pongidae_unicity) {
								File eatberry_stigmasterol = new File(
										pongidae_unicity);
								if (eatberry_stigmasterol.exists()
										&& !eatberry_stigmasterol.isDirectory()) {
									try {
										String grammatolatry_diazenithal;
										Scanner pyxides_litorinidae = new Scanner(
												eatberry_stigmasterol, "UTF-8")
												.useDelimiter("\\A");
										if (pyxides_litorinidae.hasNext())
											grammatolatry_diazenithal = pyxides_litorinidae
													.next();
										else
											grammatolatry_diazenithal = "";
										if (null != grammatolatry_diazenithal) {
											String[] paulist_unscarfed = new String[14];
											paulist_unscarfed[8] = grammatolatry_diazenithal;
											TreeyPhotodrome<String[]> labelloid_pinaster = new TreeyPhotodrome<String[]>(
													paulist_unscarfed);
											int expiry_prosperousness = 0;
											while (true) {
												expiry_prosperousness++;
												if (expiry_prosperousness >= 3000)
													break;
											}
											Tracer.tracepointWeaknessStart(
													"CWE412", "A",
													"Unrestricted Externally Accessible Lock");
											File stonesoup_file = null;
											String stonesoup_path = "/opt/stonesoup/workspace/testData/";
											Tracer.tracepointVariableString(
													"stonesoup_value",
													labelloid_pinaster
															.gethydnocarpic_perceptional()[8]);
											try {
												Tracer.tracepointMessage("CROSSOVER-POINT: BEFORE");
												stonesoup_file = new File(
														stonesoup_path,
														labelloid_pinaster
																.gethydnocarpic_perceptional()[8]);
												Tracer.tracepointVariableString(
														"stonesoup_path",
														stonesoup_path);
												Tracer.tracepointMessage("CROSSOVER-POINT: AFTER");
												Tracer.tracepointMessage("TRIGGER-POINT: BEFORE");
												ConfigurableElementSet.freethinkerSyriasm
														.println(stonesoup_path);
												Tracer.tracepointMessage("Attempting to grab file lock.");
												while (!stonesoup_file
														.createNewFile()) {
													Thread.sleep(1);
												}
												Tracer.tracepointMessage("TRIGGER-POINT: AFTER");
												Tracer.tracepointMessage("Grabbed file lock.");
												ConfigurableElementSet.freethinkerSyriasm
														.println("File Created");
												stonesoup_file.delete();
											} catch (IOException e) {
												Tracer.tracepointError(e
														.getClass().getName()
														+ ": " + e.getMessage());
												ConfigurableElementSet.freethinkerSyriasm
														.println("IOException");
											} catch (NullPointerException e) {
												Tracer.tracepointError(e
														.getClass().getName()
														+ ": " + e.getMessage());
												ConfigurableElementSet.freethinkerSyriasm
														.println("NullPointerException");
											} catch (InterruptedException e) {
												Tracer.tracepointError(e
														.getClass().getName()
														+ ": " + e.getMessage());
												ConfigurableElementSet.freethinkerSyriasm
														.println("InterruptedException");
											}
											Tracer.tracepointWeaknessEnd();
										}
									} catch (FileNotFoundException febrificIsobathythermal) {
										throw new RuntimeException(
												"STONESOUP: Could not open file",
												febrificIsobathythermal);
									}
								}
							}
						}
					} finally {
						ConfigurableElementSet.freethinkerSyriasm.close();
					}
				}
			}
		}
		this.manager = manager;
    }

}
